package com.dskj.znzn.transData.common.filter;

import cn.hutool.core.codec.Base64;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.dskj.znzn.transData.common.ThreadLocalUser;
import com.dskj.znzn.transData.common.annotaions.NeedLogin;
import com.dskj.znzn.transData.common.annotaions.NoRepeat;
import com.dskj.znzn.transData.common.base.CurrentUser;
import com.dskj.znzn.transData.common.exception.UserNotLoginExeption;
import com.dskj.znzn.transData.common.service.RedisService;
import com.dskj.znzn.transData.common.utils.IpUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.MDC;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.UUID;

/**
 * @author 魏志亮
 */
@Component
@Slf4j
public class AppInterceptor implements HandlerInterceptor {

    @Resource
    private RedisService redisService;

    private static final String BEGIN_REQUEST_TIME = "BEGIN_REQUEST_TIME";
    private static final String TRACE_ID = "TRACE_ID";

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) {

        createBase(request);

        // 如果不是映射到方法直接通过
        if (!(handler instanceof HandlerMethod)) {
            return true;
        }

        HandlerMethod handlerMethod = (HandlerMethod) handler;
        Method method = handlerMethod.getMethod();
        //是否需要登录
        loginInfo(request, method);

//        noRepeatInfo(method);

        printRequestHead(request);
        printRequestParams(request);

        return true;
    }

    public void createBase(HttpServletRequest request) {
        MDC.put(TRACE_ID, UUID.randomUUID().toString().replaceAll("-", ""));
        request.setAttribute(BEGIN_REQUEST_TIME, System.currentTimeMillis());
    }

    //TODO 暂时用不到先不实现呢
    private void noRepeatInfo(Method method) {

        if (method.isAnnotationPresent(NoRepeat.class)) {

        }
    }

    private void loginInfo(HttpServletRequest request, Method method) {
        String token = request.getHeader("token");
        CurrentUser currentUser = new CurrentUser();

        if (StringUtils.isNotEmpty(token)) {
            String userName = "";
            Integer userId = null;
            try {
                String infoBase64 = token.split("\\.")[1];
                JSONObject info = JSON.parseObject(new String(Base64.decode(infoBase64)));
                userName = info.getString("userName");
                userId = info.getInteger("userId");

            } catch (Exception e) {
                e.printStackTrace();
            }
            String result = redisService.get(String.format("zhzn_energy_system_user_login_info:%s", userName));
            if (StringUtils.isNotBlank(result) && result.equals(token)) {
                currentUser.setUserId(userId);
                currentUser.setUserName(userName);
            }
        }

        boolean loginFlag = true;
        if (method.getDeclaringClass().isAnnotationPresent(NeedLogin.class)) {
            NeedLogin parentNeedLogin = method.getDeclaringClass().getAnnotation(NeedLogin.class);
            // 类上需要登录 判断方法是否需要登录,如果不需要登录,那就不需要
            if (parentNeedLogin.required()) {
                if (method.isAnnotationPresent(NeedLogin.class)) {
                    NeedLogin needLogin = method.getAnnotation(NeedLogin.class);
                    if (!needLogin.required()) {
                        loginFlag = false;
                    }
                }
            } else {
                loginFlag = false;
                if (method.isAnnotationPresent(NeedLogin.class)) {
                    NeedLogin needLogin = method.getAnnotation(NeedLogin.class);
                    if (needLogin.required()) {
                        loginFlag = true;
                    }
                }
            }
        } else {
            //类方法没有 则直接根据方法上的进行判断
            if (method.isAnnotationPresent(NeedLogin.class)) {
                NeedLogin needLogin = method.getAnnotation(NeedLogin.class);
                loginFlag = needLogin.required();
            } else {
                loginFlag = false;
            }
        }

        if (loginFlag) {
            if (null == currentUser.getUserId()) {
                throw new UserNotLoginExeption();
            }
        }
        ThreadLocalUser.set(currentUser);
        log.info("请求的用户信息:[{}]", currentUser);
    }

    private void printRequestParams(HttpServletRequest request) {

        log.info("请求地址:[{}]", request.getRequestURI());
        log.info("请求IP:[{}]", IpUtils.getIpAddr(request));
        log.info("请求类型[{}]", request.getContentType());
        log.info("请求方式:[{}]", request.getMethod());

        String mediaType = request.getContentType();
        if (null != mediaType) {
            if (mediaType.contains(MediaType.APPLICATION_JSON_VALUE)) {
                log.info("请求BODY:[{}]", ((ParamsRequestWrapper) request).getBody());
            } else if (mediaType.contains(MediaType.APPLICATION_FORM_URLENCODED_VALUE)) {
                JSONObject argParamJson = new JSONObject();
                Enumeration params = request.getParameterNames();
                while (params.hasMoreElements()) {
                    String param = (String) params.nextElement();
                    argParamJson.put(param, request.getParameter(param));
                }
                log.info("请求PARAM:[{}]", argParamJson);
            } else {
                JSONObject argParamJson = new JSONObject();
                Enumeration params = request.getParameterNames();
                while (params.hasMoreElements()) {
                    String param = (String) params.nextElement();
                    argParamJson.put(param, request.getParameter(param));
                }
                log.info("请求PARAM:[{}]", argParamJson);
                log.info("请求FILE:[{}]", "文件类型");
            }
        }
    }


    private void printRequestHead(HttpServletRequest request) {
        JSONObject headJson = new JSONObject();
        Enumeration<String> enumeration = request.getHeaderNames();
        while (enumeration.hasMoreElements()) {
            String name = enumeration.nextElement();
            headJson.put(name, request.getHeader(name));
        }
        log.info("请求头信息[{}]", headJson);
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws Exception {
        HandlerInterceptor.super.postHandle(request, response, handler, modelAndView);
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception {
        HandlerInterceptor.super.afterCompletion(request, response, handler, ex);
        log.info("请求时长:[{}]", System.currentTimeMillis() - Long.parseLong(request.getAttribute(BEGIN_REQUEST_TIME) + ""));
        MDC.clear();
    }
}
