package com.dskj.znzn.transData.web.transConf.controller;

import com.dskj.znzn.transData.common.annotaions.NeedLogin;
import com.dskj.znzn.transData.common.base.PageInfo;
import com.dskj.znzn.transData.common.base.PageR;
import com.dskj.znzn.transData.common.base.R;
import com.dskj.znzn.transData.web.transConf.entity.TransConf;
import com.dskj.znzn.transData.web.transConf.indata.InsertOrUpdateInData;
import com.dskj.znzn.transData.web.transConf.indata.WindCodeAndTypeInData;
import com.dskj.znzn.transData.web.transConf.outdata.TransConfForPlt;
import com.dskj.znzn.transData.web.transConf.service.ITransConfService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author 魏志亮
 * @since 2024-07-08
 */
@RestController
@RequestMapping("/transConf")
@NeedLogin
@RequiredArgsConstructor(onConstructor = @__({@Autowired}))
@Api(tags = "点检表接口")
public class TransConfController {

    private final ITransConfService iTransConfService;


    @ApiOperation(value = "获取所有点检表")
    @PostMapping("/list")
    public R<List<TransConf>> list() {
        return R.ok(iTransConfService.list());
    }

    @ApiOperation(value = "获取所有点检表-分页")
    @PostMapping("/listPage")
    public PageR<TransConf> listPage(@RequestBody @Valid PageInfo info) {
        return iTransConfService.listPage(info);
    }

    @ApiOperation(value = "根据风场编号以及类型获取点检表")
    @PostMapping("/getByWindCodeAndType")
    public R<TransConf> getByWindCodeAndType(@Valid @RequestBody WindCodeAndTypeInData inData) {
        return R.ok(iTransConfService.getByWindCodeAndType(inData.getWindCode(), inData.getType()));
    }

    @ApiOperation(value = "根据风场编号以及类型获取点检表-平台查看使用")
    @PostMapping("/getByWindCodeAndTypeForPlt")
    public R<TransConfForPlt> getByWindCodeAndTypeForPlt(@Valid @RequestBody WindCodeAndTypeInData inData) {
        return R.ok(iTransConfService.getByWindCodeAndTypeForPlt(inData.getWindCode(), inData.getType()));
    }


//    @ApiOperation(value = "添加点检表")
//    @PostMapping("/add")
//    public R add(@Valid @RequestBody AddInData addInData) {
//        iTransConfService.save(addInData);
//        return R.ok();
//    }

    @ApiOperation(value = "新增或者更新点检表")
    @PostMapping("/insertOrUpdate")
    public R insertOrUpdate(@Valid @RequestBody InsertOrUpdateInData insertOrUpdateInData) {
        iTransConfService.insertOrUpdate(insertOrUpdateInData);
        return R.ok();
    }
}
